@@ -8,8 +8,6 @@
 
 using namespace node_sqlite3;
 
-Napi::FunctionReference Statement::constructor;
-
 Napi::Object Statement::Init(Napi::Env env, Napi::Object exports) {
     Napi::HandleScope scope(env);
 
@@ -23,18 +21,15 @@ Napi::Object Statement::Init(Napi::Env env, Napi::Object exports) {
       InstanceMethod("finalize", &Statement::Finalize_),
     });
 
-    constructor = Napi::Persistent(t);
-    constructor.SuppressDestruct();
-
     exports.Set("Statement", t);
     return exports;
 }
 
 // A Napi InstanceOf for Javascript Objects "Date" and "RegExp"
-bool OtherInstanceOf(Napi::Object source, char* object_type) {
-    if (object_type == "Date") {
+bool OtherInstanceOf(Napi::Object source, const char* object_type) {
+    if (strncmp(object_type, "Date", 4) == 0) {
         return source.InstanceOf(source.Env().Global().Get("Date").As<Function>());
-    } else if (object_type == "RegExp") {
+    } else if (strncmp(object_type, "RegExp", 6) == 0) {
         return source.InstanceOf(source.Env().Global().Get("RegExp").As<Function>());
     }
 
@@ -47,11 +42,10 @@ void Statement::Process() {
     }
 
     while (prepared && !locked && !queue.empty()) {
-        Call* call = queue.front();
+        std::unique_ptr<Call> call(queue.front());
         queue.pop();
 
         call->callback(call->baton);
-        delete call;
     }
 }
 
@@ -138,7 +132,7 @@ void Statement::Work_Prepare(napi_env e, void* data) {
 
     // In case preparing fails, we use a mutex to make sure we get the associated
     // error message.
-    sqlite3_mutex* mtx = sqlite3_db_mutex(baton->db->_handle);
+    STATEMENT_MUTEX(mtx);
     sqlite3_mutex_enter(mtx);
 
     stmt->status = sqlite3_prepare_v2(
@@ -158,13 +152,14 @@ void Statement::Work_Prepare(napi_env e, void* data) {
 }
 
 void Statement::Work_AfterPrepare(napi_env e, napi_status status, void* data) {
-    STATEMENT_INIT(PrepareBaton);
+    std::unique_ptr<PrepareBaton> baton(static_cast<PrepareBaton*>(data));
+    Statement* stmt = baton->stmt;
 
     Napi::Env env = stmt->Env();
     Napi::HandleScope scope(env);
 
     if (stmt->status != SQLITE_OK) {
-        Error(baton);
+        Error(baton.get());
         stmt->Finalize_();
     }
     else {
@@ -210,7 +205,13 @@ template <class T> Values::Field*
         return new Values::Float(pos, source.ToNumber().DoubleValue());
     }
     else if (source.IsObject()) {
-        std::string val = source.ToString().Utf8Value();
+        Napi::String napiVal = source.ToString();
+        // Check whether toString returned a value that is not undefined.
+        if(napiVal.Type() == 0) {
+            return NULL;
+        }
+
+        std::string val = napiVal.Utf8Value();
         return new Values::Text(pos, val.length(), val.c_str());
     }
     else {
@@ -352,20 +353,21 @@ void Statement::Work_BeginBind(Baton* baton) {
 void Statement::Work_Bind(napi_env e, void* data) {
     STATEMENT_INIT(Baton);
 
-    sqlite3_mutex* mtx = sqlite3_db_mutex(stmt->db->_handle);
+    STATEMENT_MUTEX(mtx);
     sqlite3_mutex_enter(mtx);
     stmt->Bind(baton->parameters);
     sqlite3_mutex_leave(mtx);
 }
 
 void Statement::Work_AfterBind(napi_env e, napi_status status, void* data) {
-    STATEMENT_INIT(Baton);
+    std::unique_ptr<Baton> baton(static_cast<Baton*>(data));
+    Statement* stmt = baton->stmt;
 
     Napi::Env env = stmt->Env();
     Napi::HandleScope scope(env);
 
     if (stmt->status != SQLITE_OK) {
-        Error(baton);
+        Error(baton.get());
     }
     else {
         // Fire callbacks.
@@ -404,7 +406,7 @@ void Statement::Work_Get(napi_env e, void* data) {
     STATEMENT_INIT(RowBaton);
 
     if (stmt->status != SQLITE_DONE || baton->parameters.size()) {
-        sqlite3_mutex* mtx = sqlite3_db_mutex(stmt->db->_handle);
+        STATEMENT_MUTEX(mtx);
         sqlite3_mutex_enter(mtx);
 
         if (stmt->Bind(baton->parameters)) {
@@ -425,13 +427,14 @@ void Statement::Work_Get(napi_env e, void* data) {
 }
 
 void Statement::Work_AfterGet(napi_env e, napi_status status, void* data) {
-    STATEMENT_INIT(RowBaton);
+    std::unique_ptr<RowBaton> baton(static_cast<RowBaton*>(data));
+    Statement* stmt = baton->stmt;
 
     Napi::Env env = stmt->Env();
     Napi::HandleScope scope(env);
 
     if (stmt->status != SQLITE_ROW && stmt->status != SQLITE_DONE) {
-        Error(baton);
+        Error(baton.get());
     }
     else {
         // Fire callbacks.
@@ -474,7 +477,7 @@ void Statement::Work_BeginRun(Baton* baton) {
 void Statement::Work_Run(napi_env e, void* data) {
     STATEMENT_INIT(RunBaton);
 
-    sqlite3_mutex* mtx = sqlite3_db_mutex(stmt->db->_handle);
+    STATEMENT_MUTEX(mtx);
     sqlite3_mutex_enter(mtx);
 
     // Make sure that we also reset when there are no parameters.
@@ -498,13 +501,14 @@ void Statement::Work_Run(napi_env e, void* data) {
 }
 
 void Statement::Work_AfterRun(napi_env e, napi_status status, void* data) {
-    STATEMENT_INIT(RunBaton);
+    std::unique_ptr<RunBaton> baton(static_cast<RunBaton*>(data));
+    Statement* stmt = baton->stmt;
 
     Napi::Env env = stmt->Env();
     Napi::HandleScope scope(env);
 
     if (stmt->status != SQLITE_ROW && stmt->status != SQLITE_DONE) {
-        Error(baton);
+        Error(baton.get());
     }
     else {
         // Fire callbacks.
@@ -543,7 +547,7 @@ void Statement::Work_BeginAll(Baton* baton) {
 void Statement::Work_All(napi_env e, void* data) {
     STATEMENT_INIT(RowsBaton);
 
-    sqlite3_mutex* mtx = sqlite3_db_mutex(stmt->db->_handle);
+    STATEMENT_MUTEX(mtx);
     sqlite3_mutex_enter(mtx);
 
     // Make sure that we also reset when there are no parameters.
@@ -567,13 +571,14 @@ void Statement::Work_All(napi_env e, void* data) {
 }
 
 void Statement::Work_AfterAll(napi_env e, napi_status status, void* data) {
-    STATEMENT_INIT(RowsBaton);
+    std::unique_ptr<RowsBaton> baton(static_cast<RowsBaton*>(data));
+    Statement* stmt = baton->stmt;
 
     Napi::Env env = stmt->Env();
     Napi::HandleScope scope(env);
 
     if (stmt->status != SQLITE_DONE) {
-        Error(baton);
+        Error(baton.get());
     }
     else {
         // Fire callbacks.
@@ -585,8 +590,8 @@ void Statement::Work_AfterAll(napi_env e, napi_status status, void* data) {
                 Rows::const_iterator it = baton->rows.begin();
                 Rows::const_iterator end = baton->rows.end();
                 for (int i = 0; it < end; ++it, i++) {
-                    (result).Set(i, RowToJS(env,*it));
-                    delete *it;
+                    std::unique_ptr<Row> row(*it);
+                    (result).Set(i, RowToJS(env,row.get()));
                 }
 
                 Napi::Value argv[] = { env.Null(), result };
@@ -645,7 +650,7 @@ void Statement::Work_Each(napi_env e, void* data) {
 
     Async* async = baton->async;
 
-    sqlite3_mutex* mtx = sqlite3_db_mutex(stmt->db->_handle);
+    STATEMENT_MUTEX(mtx);
 
     int retrieved = 0;
 
@@ -715,10 +720,10 @@ void Statement::AsyncEach(uv_async_t* handle) {
             Rows::const_iterator it = rows.begin();
             Rows::const_iterator end = rows.end();
             for (int i = 0; it < end; ++it, i++) {
-                argv[1] = RowToJS(env,*it);
+                std::unique_ptr<Row> row(*it);
+                argv[1] = RowToJS(env,row.get());
                 async->retrieved++;
                 TRY_CATCH_CALL(async->stmt->Value(), cb, 2, argv);
-                delete *it;
             }
         }
     }
@@ -738,13 +743,14 @@ void Statement::AsyncEach(uv_async_t* handle) {
 }
 
 void Statement::Work_AfterEach(napi_env e, napi_status status, void* data) {
-    STATEMENT_INIT(EachBaton);
+    std::unique_ptr<EachBaton> baton(static_cast<EachBaton*>(data));
+    Statement* stmt = baton->stmt;
 
     Napi::Env env = stmt->Env();
     Napi::HandleScope scope(env);
 
     if (stmt->status != SQLITE_DONE) {
-        Error(baton);
+        Error(baton.get());
     }
 
     STATEMENT_END();
@@ -774,7 +780,8 @@ void Statement::Work_Reset(napi_env e, void* data) {
 }
 
 void Statement::Work_AfterReset(napi_env e, napi_status status, void* data) {
-    STATEMENT_INIT(Baton);
+    std::unique_ptr<Baton> baton(static_cast<Baton*>(data));
+    Statement* stmt = baton->stmt;
 
     Napi::Env env = stmt->Env();
     Napi::HandleScope scope(env);
@@ -828,11 +835,15 @@ Napi::Value Statement::RowToJS(Napi::Env env, Row* row) {
 }
 
 void Statement::GetRow(Row* row, sqlite3_stmt* stmt) {
-    int rows = sqlite3_column_count(stmt);
+    int cols = sqlite3_column_count(stmt);
 
-    for (int i = 0; i < rows; i++) {
+    for (int i = 0; i < cols; i++) {
         int type = sqlite3_column_type(stmt, i);
         const char* name = sqlite3_column_name(stmt, i);
+        if (name == NULL) {
+            assert(false);
+        }
+
         switch (type) {
             case SQLITE_INTEGER: {
                 row->push_back(new Values::Integer(name, sqlite3_column_int64(stmt, i)));
@@ -870,7 +881,8 @@ Napi::Value Statement::Finalize_(const Napi::CallbackInfo& info) {
     return stmt->db->Value();
 }
 
-void Statement::Finalize_(Baton* baton) {
+void Statement::Finalize_(Baton* b) {
+    std::unique_ptr<Baton> baton(b);
     Napi::Env env = baton->stmt->Env();
     Napi::HandleScope scope(env);
 
@@ -881,8 +893,6 @@ void Statement::Finalize_(Baton* baton) {
     if (!cb.IsUndefined() && cb.IsFunction()) {
         TRY_CATCH_CALL(baton->stmt->Value(), cb, 0, NULL);
     }
-
-    delete baton;
 }
 
 void Statement::Finalize_() {
@@ -909,21 +919,17 @@ void Statement::CleanQueue() {
 
         // Clear out the queue so that this object can get GC'ed.
         while (!queue.empty()) {
-            Call* call = queue.front();
+            std::unique_ptr<Call> call(queue.front());
             queue.pop();
 
-            Napi::Function cb = call->baton->callback.Value();
+            std::unique_ptr<Baton> baton(call->baton);
+            Napi::Function cb = baton->callback.Value();
 
             if (prepared && !cb.IsEmpty() &&
                 cb.IsFunction()) {
                 TRY_CATCH_CALL(Value(), cb, 1, argv);
                 called = true;
             }
-
-            // We don't call the actual callback, so we have to make sure that
-            // the baton gets destroyed.
-            delete call->baton;
-            delete call;
         }
 
         // When we couldn't call a callback function, emit an error on the
@@ -936,12 +942,10 @@ void Statement::CleanQueue() {
     else while (!queue.empty()) {
         // Just delete all items in the queue; we already fired an event when
         // preparing the statement failed.
-        Call* call = queue.front();
+        std::unique_ptr<Call> call(queue.front());
         queue.pop();
-
         // We don't call the actual callback, so we have to make sure that
         // the baton gets destroyed.
         delete call->baton;
-        delete call;
     }
 }
